@@ -80,6 +80,7 @@ class Injected(Generic[T], metaclass=abc.ABCMeta):
             remaining_arg_names.remove("self")
         logger.info(f"partially applying {injection_targets}")
         logger.info(f"original args:{remaining_arg_names}")
+
         for injected in injection_targets.keys():
             remaining_arg_names.remove(injected)
 
@@ -88,6 +89,8 @@ class Injected(Generic[T], metaclass=abc.ABCMeta):
 
             def inner(*_args):
                 # user calls with both args or kwargs so we need to handle both.
+                assert len(_args) == len(remaining_arg_names),\
+                    f"partially applied injected function is missing some of positional args! {remaining_arg_names} for {_args}"
                 call_kwargs = dict(zip(remaining_arg_names, _args))
                 logger.info(f"partial injection call :{pformat(call_kwargs)}")
                 full_kwargs = {**kwargs, **call_kwargs}
@@ -420,6 +423,7 @@ class ZippedInjected(Injected[Tuple[A, B]]):
         return create_function(func_signature=signature, func_impl=impl)
 
 
+
 class MZippedInjected(Injected):
     def __init__(self, init_stack, *srcs: Injected):
         super().__init__(init_stack)
@@ -444,3 +448,23 @@ class MZippedInjected(Injected):
         from loguru import logger
         logger.info(f"created signature:{signature} for MZippedInjected")
         return create_function(func_signature=signature, func_impl=impl)
+
+def _injected_factory(**targets: Injected):
+    def _impl(f):
+        return Injected.partial(f, **targets)
+
+    return _impl
+
+
+def injected_factory(f):
+    """
+    any args starting with "_" is considered to be injected.
+    :param f:
+    :return:
+    """
+    sig: inspect.Signature = inspect.signature(f)
+    tgts = dict()
+    for k in sig.parameters.keys():
+        if k.startswith("_"):
+            tgts[k] = Injected.by_name(k[1:])
+    return _injected_factory(**tgts)(f)
\ No newline at end of file
