@@ -4,11 +4,10 @@ from copy import copy
 from dataclasses import dataclass, field, replace
 from functools import wraps
 from itertools import chain
-from pickle import PicklingError
 from typing import Union, Type, TypeVar, Callable, Dict, Any
 
-import pinject
 import cloudpickle
+import pinject
 from cytoolz import merge, valmap
 from expression import Some, Nothing
 from expression.collections import Map
@@ -17,13 +16,12 @@ from makefun import create_function, wraps
 from pampy import match
 from pinject import BindingSpec
 from pinject.scoping import SingletonScope
-from returns.result import safe, Failure, Result
-from tabulate import tabulate
-
 from pinject_design.di.design import Bind, FunctionProvider, ProviderTrait, InjectedProvider, PinjectConfigure, \
     PinjectProvider, ensure_self_arg, PinjectBind
 from pinject_design.di.graph import ExtendedObjectGraph
 from pinject_design.di.injected import Injected
+from returns.result import safe, Failure
+from tabulate import tabulate
 
 prototype = pinject.provides(in_scope=pinject.PROTOTYPE)
 # is it possible to create a binding class which also has an ability to dynamically add..?
@@ -266,13 +264,19 @@ class Design:
             x = x.bind(k).to_instance(v)
         return x
 
-    def bind_provider(self, **kwargs):
+    def bind_provider(self, **kwargs: Union[Callable, Injected]):
         x = self
         for k, v in kwargs.items():
             # logger.info(f"binding provider:{k}=>{v}")
             if isinstance(v, type):
                 logger.warning(f"{k}->{v}: class is used for bind_provider. fixing automatically.")
                 x = x.bind(k).to_class(v)
+            if isinstance(v, Injected):
+                x = x.bind(k).to_provider(v)
+            elif not callable(v):
+                logger.warning(
+                    f"{k}->{v}: non-callable or non-injected is passed to bind_provider. fixing automatically.")
+                x = x.bind(k).to_instance(v)
             else:
                 x = x.bind(k).to_provider(v)
         return x
